#define INITGUID  // THIS LINE REQUIRED FOR QueryInterface() CALL!

#include <mmsystem.h>
#include <dsound.h>
#include "util.cpp"

void CALLBACK DirSndTimerProc(HWND, UINT, UINT, DWORD);

/***** TYPEDEFs ***********************************************/
typedef struct _FileInfo
{
  UINT BytesToEndOfFile;
  UINT DataChunkSize;
  UINT DataChunkOffset;
} FILEINFO;


/***** C++ CLASSES ********************************************/
class DirSnd
{
  private:
	// PRIVATE VARIABLES
	BOOL     m_LoopingStatus,  // 0 = play once, 1 = infinite play loop.
		     m_WaitForEnd;
	char     m_Filename[_MAX_PATH];
	DWORD    m_dwCoopLevel,
		     m_PreviousPlayCursor,
			 m_TotalBytesPlayed;
	FILEINFO m_FileInfo;
	GUID    *m_pGuid;
	HMMIO    m_hMmioFile;
	int      m_CreateErrorCode;

	// PRIVATE METHODS
	void CloseWaveFile(void);
	void InitialBufferLoad(void);
	int  OpenWaveFile(void);
	void ResetFile(void);

  public:
	// PUBLIC VARIABLES
    DSBUFFERDESC          m_DSBufDesc;
	HWND                  m_hwnd;
    LPDIRECTSOUND         m_pDS;
    LPDIRECTSOUNDBUFFER   m_pDSBuffer;
	LPDIRECTSOUND3DBUFFER m_pDS3DBuffer;
    UINT                  m_TimerID;

    // PUBLIC METHODS
	DirSnd* Duplicate(void);
    int     Create(DWORD, DWORD, HWND, GUID*, UINT, UINT, char*);
    void    Destroy(void);
	void    Play(BOOL);
	void    Stop(void);
    void    UpdateStreamBuffer(void);
};


/*************************************************************************
*
* FUNCTION: DirSnd::Create()
*
* DESCRIPTION: Creates a Direct Sound object and SECONDARY buffer based
*              upon the parameters passed to this function.  The buffer
*              is then filled with WAVE data from the WAVE file.  Do
*              NOT use this function to create a PRIMARY buffer!
*
*              To create a PRIMARY buffer, #include the file
*              "ds3dlstr.cpp", declare a single (only one)
*              "Listener3D" object, and call the Listener3D::Create()
*              method.
*
* DANGER:      The dwFlags parameter passed to this function MUST NOT
*              include the DSBCAPS_PRIMARYBUFFER flag!  Setting the
*              dwFlags parameter to DSBCAPS_PRIMARYBUFFER will cause
*              problems!
*
* RETURN VALUE: 0 = NO ERRORS.
*               1 = DirectSoundCreate() fails.
*               2 = SetCooperativeLevel() fails.
*               3 = OpenWaveFile() fails.
*               4 = CreateSoundBuffer() fails.
*               5 = QueryInterface() fails, m_pDS3DBuffer not created.
*
*************************************************************************/
int DirSnd::Create(DWORD dwFlags, DWORD CoopLevel, HWND hwnd,
				   GUID *PtrGuid, UINT SizeOfBuf, UINT Timer_ID,
				   char *NameOfFile)
{
  // INITIALIZE CLASS MEMBER VARIABLES.
  m_DSBufDesc.dwFlags  = dwFlags;
  m_dwCoopLevel        = CoopLevel;
  m_hwnd               = hwnd;
  m_pDS3DBuffer        = 0;
  m_pGuid              = PtrGuid;
  m_PreviousPlayCursor = 0;
  m_TimerID            = Timer_ID;
  m_TotalBytesPlayed   = 0;
  m_WaitForEnd         = FALSE;
  strcpy(m_Filename, NameOfFile);

  
  // CREATE DIRECT SOUND OBJECT.
  if (DirectSoundCreate(m_pGuid, &m_pDS, NULL) == DS_OK)
  {
    // SET COOP LEVEL
    if (m_pDS->SetCooperativeLevel(m_hwnd, m_dwCoopLevel) != DS_OK)
    {
	  return(m_CreateErrorCode = 2);
    }
  }
  else
  {
	return(m_CreateErrorCode = 1);
  }


  if (OpenWaveFile())
  {
	// OpenWaveFile FAILS!
	Box(NULL, "Direct Sound Failure", "OpenWaveFile()  Fails");
    return(m_CreateErrorCode = 3);
  }

  
  if (!(m_DSBufDesc.dwFlags & DSBCAPS_STATIC))
  {
	// Buffer is STREAMING buffer.
	// Determine proper size of the STREAMING buffer.  The Buffer
	// size--whether static or streaming--is stored in dwBufferBytes.
    //
    // NOTE: dwBufferBytes is first initialized in OpenWaveFile(),
	//       where it is initialized to the size of the file's
	//       data chunk.
    if (SizeOfBuf == NULL)
    {
	  // By default, make buffer 2 seconds long.
      m_DSBufDesc.dwBufferBytes = 2 * m_DSBufDesc.lpwfxFormat->nAvgBytesPerSec;
    }
    else
    {
	  // Buffer size is requested by caller.
	  m_DSBufDesc.dwBufferBytes = SizeOfBuf;
    }

    if (m_DSBufDesc.dwBufferBytes > m_FileInfo.DataChunkSize)
    {
      // The buffer is larger than the data chunk to be played.
	  // So set dwBufferBytes to size of data chunk.
      m_DSBufDesc.dwBufferBytes = m_FileInfo.DataChunkSize;
    }
  }


  // Create Direct Sound Buffer.
  if (m_pDS->CreateSoundBuffer(&m_DSBufDesc, &m_pDSBuffer, NULL) != DS_OK)
  {
	return(m_CreateErrorCode = 4);
  }


  if (dwFlags & DSBCAPS_CTRL3D)
  {
	// Initialize m_pDS3DBuffer so IDirect3DSoundBuffer
	// API functions can be called via m_pDS3DBuffer.
	m_pDSBuffer->QueryInterface(IID_IDirectSound3DBuffer,
		                        (void **) &m_pDS3DBuffer);
	if (m_pDS3DBuffer == NULL)
    {
	  return(m_CreateErrorCode = 5);
    }
  }

  
  // Load the entire buffer with data.
  InitialBufferLoad();

  // NOTE: If buffer is a streaming buffer, leave the WAVE file
  //       open--it will be closed in the Destroy() function.
  if (m_DSBufDesc.dwFlags & DSBCAPS_STATIC)
  {
	// Buffer is static.  All wave data has been loaded into
	// the buffer, so close the wave file.
	CloseWaveFile();
  }

  return(m_CreateErrorCode = 0);
}


/*************************************************************************
*
* FUNCTION: DirSnd::Destroy()
*
* DESCRIPTION: This function is the complimentary function to
*              DirSnd::Create().  For each DirSnd object created,
*              DirSnd::Destroy() MUST be called before the program
*              terminates to perform cleanup operations.
*
*              Which cleanup operations need to be perfomed depends
*              upon the value of m_CreateErrorCode, which was assigned
*              a value in DirSnd::Create().
*
*************************************************************************/
void DirSnd::Destroy()
{
  if (!(m_DSBufDesc.dwFlags & DSBCAPS_STATIC))
  {
	// Buffer is a STREAMING buffer.
	if (m_CreateErrorCode == 0)
	{
	  KillTimer(m_hwnd, m_TimerID);
	}

	if (m_CreateErrorCode == 0 || m_CreateErrorCode > 3)
	{
	  // Close wave file ONLY for streaming buffers.
	  CloseWaveFile();
	}
  }

  if (m_CreateErrorCode == 0 || m_CreateErrorCode > 3)
  {
	// Space for m_DSBufDesc.lpwfxFormat is allocated in OpenWaveFile().
	delete(m_DSBufDesc.lpwfxFormat);  // Deallocate WAVEFORMATEX space.
  }


  // RELEASE DIRECT SOUND BUFFER(S) AND DIRECT SOUND OBJECT.
  if (m_pDS3DBuffer)
  {
    // Release the IDirect3DSoundBuffer, if it was created.
    m_pDS3DBuffer->Release();
  }

  if (m_CreateErrorCode == 0 || m_CreateErrorCode > 4)
  {
	// Release the IDirectSoundBuffer.
    m_pDSBuffer->Release();
  }
	
  if (m_CreateErrorCode != 1)
  {
	// Release the Direct Sound object.
    m_pDS->Release();
  }

  return;
}


/*************************************************************************
*
* FUNCTION: DirSnd::Duplicate()
*
* DESCRIPTION: This function will create a DirSnd object that uses the
*              same Direct Sound object and STATIC buffer as the object
*              that calls this function.
*
*              NOTE: Duplicate() will NOT work for streaming buffers
*                    because the contents of streaming buffers
*                    continually change, making it impossible for
*                    a duplicate buffer to play the end of a long
*                    file while the original buffer plays the beginning
*                    of that file.
*
* RETURN VALUE: If successful, this function returns a pointer to
*               a newly allocated and initialized DirSnd object.
*               If this function fails, it returns NULL.
*
*************************************************************************/
DirSnd* DirSnd::Duplicate()
{
  DirSnd *pDirSnd;

  if (!(m_DSBufDesc.dwFlags & DSBCAPS_STATIC))
  {
	// Buffer is NOT static.
	// Duplicate() works ONLY for STATIC buffers.
	return(NULL);
  }

  // Allocate a new DirSnd object.
  pDirSnd = new(DirSnd);

  if (pDirSnd == NULL)
  {
	return(NULL);
  }

  // Copy class variables from this object into the new DirSnd object.
  memcpy(pDirSnd, this, sizeof(DirSnd));

  // Create the duplicate Direct Sound buffer.
  if (m_pDS->DuplicateSoundBuffer(m_pDSBuffer, &(pDirSnd->m_pDSBuffer))
	  == DS_OK)
  {
    return(pDirSnd);
  }
  
  // IDirectSound::DuplicateSoundBuffer() FAILED.
  delete(pDirSnd);

  return(NULL);
}


/*************************************************************************
*
* FUNCTION: DirSnd::Play()
*
* DESCRIPTION: Begin playing the file.
*
* PARAMETERS: If the parameter passed is FALSE (zero), the file will
*             be played only ONE TIME.
*
*             If the parameter passed is TRUE (non-zero), the file
*             will be played in a continuous loop.
*
*************************************************************************/
void DirSnd::Play(BOOL Looping_Status)
{
  DWORD BytesToLock = 0,
	    CurrentPlayCursor,
		CurrentWriteCursor;
  UINT  TimerLength;
  LONG  Volume;

  // m_LoopingStatus exists so other class functions can
  // determiine if the file is looping or not.
  m_LoopingStatus = Looping_Status;

  m_pDSBuffer->GetVolume(&Volume);  // Save volume setting.
  m_pDSBuffer->SetVolume(-10000);   // Set volume to minimum.

  if (m_DSBufDesc.dwFlags & DSBCAPS_STATIC)
  {
	// Buffer is a STATIC buffer.
	if (m_LoopingStatus)
	{
	  m_pDSBuffer->Play(0, 0, DSBPLAY_LOOPING);
	}
	else
	{
	  m_pDSBuffer->Play(0, 0, 0);
	}
  }
  else
  {
	// Buffer is a STREAMING buffer.
	if (m_WaitForEnd)
	{
	  // Buffer is NOT in looping mode AND the end of file has been
	  // loaded into buffer when Stop() was called.  Calculate
	  // TimerLength so the timer will be triggered precisely after
	  // the file is done playing.
	  m_pDSBuffer->GetCurrentPosition(&CurrentPlayCursor,
		                              &CurrentWriteCursor);
      if (CurrentPlayCursor > m_PreviousPlayCursor)
      {
        BytesToLock = (CurrentPlayCursor - m_PreviousPlayCursor);
      }
      else if (CurrentPlayCursor < m_PreviousPlayCursor)
      {
	    BytesToLock = m_DSBufDesc.dwBufferBytes -
		              (m_PreviousPlayCursor - CurrentPlayCursor);
      }

	  m_TotalBytesPlayed += BytesToLock;
      TimerLength = (DWORD) (float(m_FileInfo.DataChunkSize -
		                     m_TotalBytesPlayed) /
						     float(m_DSBufDesc.lpwfxFormat->nAvgBytesPerSec)
						     * 1000.0);
      if (TimerLength < 121)
	  {
 	    // File is < 121 mSec from done playing.  Terminate playing
	    // and reload the buffer so it's ready to play again.
	    InitialBufferLoad();
		m_pDSBuffer->SetVolume(Volume);   // Restore volume setting.
		return;
	  }
	}
	else
	{
      // Set TimerLength so timer triggers after 1/2 a buffer is played.
      TimerLength = (UINT)  (((float(m_DSBufDesc.dwBufferBytes) /
		                       float(m_DSBufDesc.lpwfxFormat->nAvgBytesPerSec))
		                     * 1000.0) / 2.0);
	}

	SetTimer(m_hwnd, m_TimerID, TimerLength, DirSndTimerProc);
	m_pDSBuffer->Play(0, 0, DSBPLAY_LOOPING);
  }

  m_pDSBuffer->SetVolume(Volume);   // Restore volume setting.

  return;
}


/*************************************************************************
*
* FUNCTION: DirSnd::Stop()
*
* DESCRIPTION: Stops the file from playing.
*
*************************************************************************/
void DirSnd::Stop()
{
  if (!(m_DSBufDesc.dwFlags & DSBCAPS_STATIC))
  {
	// Buffer is STREAMING buffer.
    KillTimer(m_hwnd, m_TimerID);
  }
  
  m_pDSBuffer->Stop();
  return;
}


/*************************************************************************
*
* FUNCTION: DirSnd::InitialBufferLoad()
*
* DESCRIPTION: This function loads the entire buffer with data from the
*              beginning of the file.  This function must be called
*              before a file can be played.
*
*              This function is called under 2 circumstances:
*
*              1) This function is called whenever a buffer is
*                 created (when a DirSnd class is declared),
*                 whether the buffer is STATIC or STREAMING.
*
*              2) This function is called whenever a STREAMING buffer
*                 is used AND the entire file finishes playing AND the
*                 m_LoopingStatus flag is TRUE.  This ensures that
*                 the STREAMING buffer is ready to be played again from
*                 the start of the file the next time Play() is called.
*
*************************************************************************/
void DirSnd::InitialBufferLoad()
{
  DWORD  Length1,
         Length2;
  LPVOID pData1,
         pData2;
  LONG   Volume;

  m_pDSBuffer->GetVolume(&Volume);  // Save volume setting.
  m_pDSBuffer->SetVolume(-10000);   // Set volume to minimum.
  m_pDSBuffer->Stop();
  m_pDSBuffer->SetCurrentPosition(0);
  m_pDSBuffer->SetVolume(Volume);   // Restore volume setting.
  ResetFile();

  if (!(m_DSBufDesc.dwFlags & DSBCAPS_STATIC))
  {
	// Buffer is a STREAMING buffer.  Reset some class variables.
	KillTimer(m_hwnd, m_TimerID);
    m_PreviousPlayCursor = 0;
    m_TotalBytesPlayed   = 0;
    m_WaitForEnd         = FALSE;
  }

  // LOCK THE DIRECT SOUND BUFFER BEFORE LOADING IT WITH DATA.
  if (m_pDSBuffer->Lock(0, m_DSBufDesc.dwBufferBytes, &pData1, &Length1,
		                &pData2, &Length2, NULL) != DS_OK)
  {
    Box(NULL, "FAILURE", "Lock() FAILS");
  }

  // Load Direct Sound buffer with WAVE data from file.
  if (mmioRead(m_hMmioFile, (char *) pData1, m_DSBufDesc.dwBufferBytes) == -1)
  {
    Box(NULL, "FAILURE", "mmioRead() FAILS.");
  }


  // UNLOCK DIRECT SOUND BUFFER IMMEDIATELY AFTER WRITING TO IT.
  if (m_pDSBuffer->Unlock(pData1, Length1, pData2, Length2) != DS_OK)
  {
    Box(NULL, "FAILURE", "Lock() FAILS");
  }

  m_FileInfo.BytesToEndOfFile = m_FileInfo.DataChunkSize -
	                            m_DSBufDesc.dwBufferBytes;

  return;
}


/*************************************************************************
*
* FUNCTION: DirSnd::UpdateStreamBuffer()
*
* DESCRIPTION: This function is called ONLY for STREAMING buffers!
*
*              NEVER call this function directly!  This function is
*              called ONLY by DirSndTimerProc(), which gets called
*              automatically when the Windows timer is triggered.
*
*              Whenever the Windows timer is triggered via a DirSnd
*              object, this function will be called and data will be
*              loaded from the file into the buffer.  For details,
*              see commented code below.
*
*************************************************************************/
void DirSnd::UpdateStreamBuffer()
{
  char  *pData1,
        *pData2;		 
  DWORD  BytesToLock,
	     BytesToRead,
		 CurrentPlayCursor,
		 CurrentWriteCursor,
	     Length1,
         Length2;

  if (m_WaitForEnd)
  {
    // The ENTIRE non-looping file has JUST FINISHED PLAYING!
	// Terminate playing and reload the buffer so it's ready to
	// play again.
    InitialBufferLoad();
    return;
  }


  // Determine how many buffer bytes are available for writing.
  m_pDSBuffer->GetCurrentPosition(&CurrentPlayCursor, &CurrentWriteCursor);

  if (CurrentPlayCursor > m_PreviousPlayCursor)
  {
    BytesToLock = (CurrentPlayCursor - m_PreviousPlayCursor);
  }
  else if (CurrentPlayCursor < m_PreviousPlayCursor)
  {
	BytesToLock = m_DSBufDesc.dwBufferBytes -
		          (m_PreviousPlayCursor - CurrentPlayCursor);
  }
  else
  {
	// CurrentPlayCursor == m_PreviousPlayCursor
	return;
  }


  // LOCK THE DIRECT SOUND BUFFER BEFORE LOADING IT WITH DATA.
  if (m_pDSBuffer->Lock(m_PreviousPlayCursor, BytesToLock, &pData1, &Length1,
	                    &pData2, &Length2, 0) != DS_OK)
  {
    Box(NULL, "FAILURE", "Lock() FAILS");
  }

  // Update m_PreviousPlayCursor
  m_PreviousPlayCursor = CurrentPlayCursor;


  /***** Update pData1/Length1 part of buffer. **************************/
  /**********************************************************************/
  if (Length1 <= m_FileInfo.BytesToEndOfFile)
  {
    BytesToRead = Length1;
    m_FileInfo.BytesToEndOfFile -= Length1;
  }
  else
  {
    BytesToRead = m_FileInfo.BytesToEndOfFile;
    m_FileInfo.BytesToEndOfFile = 0;
  }


  // Read data from file into buffer.
  if (mmioRead(m_hMmioFile, pData1, BytesToRead) == -1)
  {
    Box(NULL, "FAILURE", "mmioRead() FAILS.");
  }


  if (!m_LoopingStatus)
  {
	// Keep track of how many bytes have been played so it can be
	// determined when the end of the file is near.  m_TotalBytesPlayed
	// will be used to reprogram the timer so the timer will next
	// trigger precisely when the file has finished playing.
    m_TotalBytesPlayed += BytesToLock;

    if (m_FileInfo.BytesToEndOfFile == 0)
    {
      // All bytes have been loaded into buffer.  Now we
	  // must wait for the end of the file to be played.
      m_WaitForEnd = TRUE;
    }
  }

  
  if (m_LoopingStatus && (Length1 - BytesToRead) > 0)
  {
	// File will repeat playing AND the end of file has been reached.
	// Seek to beginning of file and continue filling the buffer.
    ResetFile();

	if (mmioRead(m_hMmioFile, pData1 + BytesToRead, Length1 - BytesToRead)
		== -1)
    {
      Box(NULL, "FAILURE", "mmioRead() FAILS.");
    }
    m_FileInfo.BytesToEndOfFile = m_FileInfo.DataChunkSize -
		                          (Length1 - BytesToRead);
  }

  if (m_FileInfo.BytesToEndOfFile == 0)
  {
    m_FileInfo.BytesToEndOfFile = m_FileInfo.DataChunkSize;
    ResetFile();  // Seek to beginning of data chunk.
  }


  /***** If necessary, update pData2/Length2 part of buffer. ************/
  /**********************************************************************/
  if (pData2)
  {
    if (Length2 <= m_FileInfo.BytesToEndOfFile)
    {
	  BytesToRead = Length2;
  	  m_FileInfo.BytesToEndOfFile -= Length2;
    }
    else
    {
      BytesToRead = m_FileInfo.BytesToEndOfFile;
	  m_FileInfo.BytesToEndOfFile = 0;
    }


    if (mmioRead(m_hMmioFile, pData2, BytesToRead) == -1)
    {
      Box(NULL, "FAILURE", "mmioRead() FAILS.");
    }


	if (!m_LoopingStatus && (m_FileInfo.BytesToEndOfFile == 0))
    {
      // All bytes have been loaded into buffer.  Now we
      // must wait for the end of the file to be played.
      m_WaitForEnd = TRUE;
    }

	
    if (m_LoopingStatus && (Length2 - BytesToRead) > 0)
    {
	  // File will repeat playing AND the end of file has been reached.
	  // Seek to beginning of file and continue filling the buffer.
      ResetFile();

	  if (mmioRead(m_hMmioFile, pData2 + BytesToRead, Length2 - BytesToRead)
		  == -1)
      {
        Box(NULL, "FAILURE", "mmioRead() FAILS.");
      }
	  m_FileInfo.BytesToEndOfFile = m_FileInfo.DataChunkSize -
		                            (Length2 - BytesToRead);
    }
  }  // End: if (pData2)


  // UNLOCK DIRECT SOUND BUFFER IMMEDIATELY AFTER WRITING TO IT.
  if (m_pDSBuffer->Unlock(pData1, Length1, pData2, Length2) != DS_OK)
  {
    Box(NULL, "FAILURE", "Lock() FAILS");
  }


  if (m_FileInfo.BytesToEndOfFile == 0)
  {
    m_FileInfo.BytesToEndOfFile = m_FileInfo.DataChunkSize;
    ResetFile();  // Seek to beginning of data chunk.
  }


  /**********************************************************************/
  /**********************************************************************/
  if (m_WaitForEnd)
  {
	// To get here, ALL the file bytes have been loaded into the buffer
	// AND the m_LoopingStatus flag is FALSE.
	UINT TimerLength;
    
    // Get the latest m_TotalBytesPlayed count.
    m_pDSBuffer->GetCurrentPosition(&CurrentPlayCursor,
		                            &CurrentWriteCursor);
    if (CurrentPlayCursor > m_PreviousPlayCursor)
    {
      m_TotalBytesPlayed += (CurrentPlayCursor - m_PreviousPlayCursor);
    }
    else if (CurrentPlayCursor < m_PreviousPlayCursor)
    {
	  m_TotalBytesPlayed += (m_DSBufDesc.dwBufferBytes -
		                     (m_PreviousPlayCursor - CurrentPlayCursor));
    }

	// Reprogram the timer so it triggers precisely
	// when the file is done playing.
    TimerLength = (DWORD) (float(m_FileInfo.DataChunkSize -
		                   m_TotalBytesPlayed) /
						   float(m_DSBufDesc.lpwfxFormat->nAvgBytesPerSec)
						   * 1000.0);
	if (TimerLength > 120)
	{
	  // Reprogram timer only if there is > 120ms left to play.
	  // Playing of the file will be terminated at the top of this
	  // function next time the timer is triggered.
	  KillTimer(m_hwnd, m_TimerID);
	  SetTimer(m_hwnd, m_TimerID, TimerLength, DirSndTimerProc);
	}
    else
	{
	  // File is virtually done playing.  Terminate playing and
	  // reload the buffer so it's ready to play again.
	  InitialBufferLoad();
	}
  }  // End: if (m_WaitForEnd)

  return;
}


/*************************************************************************
*
* FUNCTION: DirSnd::OpenWaveFile()
*
* DESCRIPTION: Opens a WAVE file and initializes some m_DSBufDesc fields
*              from information stored in the WAVE file's header.
*
* RETURN VALUE: 0 = NO ERRORS.
*               1 = Cannot find/open DirSnd.m_Filename
*               2 = Can't find "WAVE" chunk.
*               3 = Can't find "fmt " chunk.
*               4 = mmioRead() fails.
*               5 = Can't find "data" chunk.
*
*************************************************************************/
int DirSnd::OpenWaveFile()
{
  MMCKINFO ParentChunk,
           SubChunk;

  // Open the WAVE file.
  m_hMmioFile = mmioOpen(m_Filename, NULL, MMIO_READ);
  if(!m_hMmioFile)
  {
	return(1);
  }

  // Verify that file is a WAVE file.
  ParentChunk.fccType = mmioFOURCC('W', 'A', 'V', 'E');
  if (mmioDescend(m_hMmioFile, &ParentChunk, NULL, MMIO_FINDRIFF))
  {
	CloseWaveFile();
	return(2);
  }

  // Find the "fmt " chunk.
  SubChunk.ckid = mmioFOURCC('f', 'm', 't', ' ');
  if (mmioDescend(m_hMmioFile, &SubChunk, &ParentChunk, MMIO_FINDCHUNK))
  {
	CloseWaveFile();
	return(3);
  }
    
  // Allocate memory so m_DSBufDesc.lpwfxFormat points to WAVEFORMATEX struct.
  m_DSBufDesc.lpwfxFormat = new(WAVEFORMATEX);

  // Read PCM wave format data from file into WAVEFORMATEX struct.
  // NOTE: Read ONLY sizeof(PCMWAVEFORMAT) into m_DSBufDesc.lpwfxFormat!!!
  if (mmioRead(m_hMmioFile, (char *) m_DSBufDesc.lpwfxFormat,
	  sizeof(PCMWAVEFORMAT)) == -1)
  {
	delete(m_DSBufDesc.lpwfxFormat);
	CloseWaveFile();
	return(4);
  }

  // Get size of data chunk--returned in SubChunk.cksize.
  SubChunk.ckid = mmioFOURCC('d', 'a', 't', 'a');
  if (mmioDescend(m_hMmioFile, &SubChunk, &SubChunk, MMIO_FINDCHUNK))
  {
	delete(m_DSBufDesc.lpwfxFormat);
	CloseWaveFile();
	return(5);
  }

  // Initialize DirSnd class variables..
  m_DSBufDesc.dwSize         = sizeof(DSBUFFERDESC);
  m_DSBufDesc.dwBufferBytes  = SubChunk.cksize; // Size of DS Buffer.
  m_FileInfo.DataChunkSize   = SubChunk.cksize; // Size of "data" chunk.
  m_FileInfo.DataChunkOffset = SubChunk.dwDataOffset;
  m_DSBufDesc.lpwfxFormat->cbSize = 0; // Init. last WAVEFORMATEX field.

  return(0);
}


/*************************************************************************
*
* FUNCTION: DirSnd::CloseWaveFile()
*
*************************************************************************/
void DirSnd::CloseWaveFile()
{
  mmioClose(m_hMmioFile, NULL);
  return;
}


/*************************************************************************
*
* FUNCTION: DirSnd::ResetFile()
*
*************************************************************************/
void DirSnd::ResetFile()
{
  if (mmioSeek(m_hMmioFile, m_FileInfo.DataChunkOffset, SEEK_SET) == -1)
  {
    Box(NULL, "FAILURE", "mmioSeek() FAILS.");
  }
  return;
}